import setuptools

def get_description():
	# Get long description.

	with open( 'README.md', 'r' ) as f:
		desc = f.read()
	return desc

def get_requirements( req ):
	# Load list of dependencies.

	install_requires = []
	with open( req ) as f:
		for line in f:
			if not line.startswith( '#' ):
				install_requires.append( line.strip() )
	return install_requires

setuptools.setup(
	name = 'pymdown-csv2md',
	version = '0.2.0',
	author = 'Max Hegler',
	author_email = 'maxghegler@gmail.com',
	description = 'A Python-Markdown extension for converting .csv file to markdown tables inline.',
	long_description = get_description(),
	long_description_content_type = 'text/markdown',
	url = 'https://github.com/mghweb/pymdown-csv2md',
	packages = setuptools.find_packages(),
	license = 'MIT License',
	classifiers = [
		'Intended Audience :: Developers',
		'Programming Language :: Python :: 3',
		'License :: OSI Approved :: MIT License',
		'Operating System :: OS Independent',
		'Topic :: Software Development :: Libraries :: Python Modules',
		'Topic :: Text Processing :: Filters',
		'Topic :: Text Processing :: Markup :: HTML'
	],
	python_requires = '>3.0',
	install_requires = get_requirements( 'requirements.txt' ),
	entry_points = {
		'markdown.extensions': [ 'csv2md = package.csv2md.csv2md:Csv2MdExtension' ]
	}
)


import markdown
import os
from ..csv2md.csv2md import Csv2MdExtension

CURRENT_DIR = os.path.abspath(os.path.dirname(__file__))

class TestMarkdown():
	def __init__(self, *args, **kwargs):

		self.md = markdown.Markdown(extensions = [ Csv2MdExtension(), 'pymdownx.extra' ], extension_configs = {})

		for subdir, dirs, files in os.walk(os.path.join(CURRENT_DIR, 'fixtures')):
			for file in files:
				if (file.endswith('.md')):
					self.md.convertFile(input = os.path.join(CURRENT_DIR, 'fixtures', file), output = os.path.join(CURRENT_DIR, 'fixtures', (file + '.html')))

# run test
test = TestMarkdown()


from markdown.extensions import Extension
from markdown.preprocessors import Preprocessor
from mdtable import MDTable
import logging
import re
import os

RE_CSV_FILE_INCLUDE = r'@csv"([^\s"]+\.csv)"'

logging.basicConfig()
LOGGER_NAME = 'csv2md'
log = logging.getLogger(LOGGER_NAME)

class Cvs2MdProcessor(Preprocessor):
	def __init__(self, md, config):
		super(Cvs2MdProcessor, self).__init__(md)

		# Compile regex
		self.compiled_regex = re.compile(RE_CSV_FILE_INCLUDE)

		# Create cache
		self.cache = {}

		# Set Config
		self.base_path = config['base_path'][0]
		self.aligns = config['aligns'][0]
		self.padding = config['padding'][0]
		self.delimiter = config['delimiter'][0]
		self.quotechar = config['quotechar'][0]
		self.escapechar = config['escapechar'][0]
	
	def resolve_file_path(self, file_path):
		return os.path.normpath(os.path.join(self.base_path, os.path.expanduser(file_path)))
	
	def get_table_lines(self, file_path):
		table_lines = []
		try:
			file_path = self.resolve_file_path(file_path)
			if file_path in self.cache:
				table_lines = self.cache[file_path]
			else:
				markdown_table = MDTable(file_path, self.aligns, self.padding, self.delimiter, self.quotechar, self.escapechar)
				table = markdown_table.get_table()
				table_lines = table.splitlines()
				table_lines.append('\n')
				self.cache[file_path] = table_lines
		except Exception as e:
			log.exception(' Could not find file: {}'.format(file_path))
		return table_lines

	def run(self, lines = []):
		new_lines = []
		for line in lines:
			total_result = []
			offset = 0
			matches = self.compiled_regex.finditer(line)
			for match in matches:
				total_match = match.group(0)

				# Get table text
				result = self.get_table_lines(match.group(1))

				# Output result to total_result
				if len(result) == 0:
					result = [total_match]
				else:
					total_result = result

			# All replacements are done, copy the rest of the string
			if not total_result:
				total_result.append(line[offset:])
				
			new_lines.extend(total_result)

		return new_lines

class Csv2MdExtension(Extension):
	def __init__(self, *args, **kwargs):

		self.config = {
			'base_path': [
				'.',
				'Base path from where relative paths are calculated.'
			],
			'aligns': [
				None,
				'Tuple of alignments, must have same number as number of columns.'
			],
			'padding': [
				1,
				'Padding to use in raw formatted markdown table.'
			],
			'delimiter':  [
				",",
				'Delimiter character in CSV file.'
			],
			'quotechar': [
				'"',
				'Quote character in CSV file.'
			],
			'escapechar':  [
				"",
				'Escape character in CSV file.'
			]
		}

		super().__init__(*args, **kwargs)
	
	def extendMarkdown(self, md):
		md.registerExtension(self)
		md.preprocessors.register(Cvs2MdProcessor(md, self.config), 'csv2md', 101)
		

def makeExtension(*args, **kwargs):
	return Csv2MdExtension(*args, **kwargs)
